@@ -1,7 +1,7 @@
 package MooseX::Storage::Engine::Trait::WithRoles;
 our $AUTHORITY = 'cpan:YANICK';
 # ABSTRACT: An engine trait to include roles in serialization
-$MooseX::Storage::Engine::Trait::WithRoles::VERSION = '0.1.0';
+$MooseX::Storage::Engine::Trait::WithRoles::VERSION = '0.2.0';
 use Moose::Util qw/ with_traits /;
 
 use List::Util qw/ pairgrep /;
@@ -17,17 +17,20 @@ around collapse_object => sub {
 
     my $packed = $orig->( $self, @args );
 
-    if( my @roles = map { $_->name } @{ $self->object->meta->roles } ) {
-        $packed->{'__ROLES__'} = [
-            apply { 
-                $_ = { $_->meta->genitor->name => { pairgrep { $a ne '<<MOP>>' }  %{ $_->meta->parameters } } }
-                    if $_->meta->isa('MooseX::Role::Parameterized::Meta::Role::Parameterized') 
-            } @roles
-        ];
+    my @extra;
+    ( $packed->{'__CLASS__'}, @extra ) = split '\|', ($self->object->meta->superclasses)[0]
+        if $self->object->meta->is_anon_class or $self->object->meta->name =~ /__ANON__/ ;
+
+    my %in_superclass = map { $_ => 1 } map { split '\|', $_->name } @{ $packed->{'__CLASS__'}->meta->roles };
+
+    if( my @roles = grep { !$in_superclass{$_} } map { split '\|', } ( @extra, map { $_->name } @{ $self->object->meta->roles } ) ) {
+        @roles = apply { 
+            $_ = { $_->meta->genitor->name => { pairgrep { $a ne '<<MOP>>' }  %{ $_->meta->parameters } } }
+                if $_->meta->isa('MooseX::Role::Parameterized::Meta::Role::Parameterized') 
+        } @roles;
+        $packed->{'__ROLES__'} = \@roles;
     }
 
-    ( $packed->{'__CLASS__'} ) = $self->object->meta->superclasses
-        if $self->object->meta->is_anon_class;
 
     return $packed;
 };
@@ -56,7 +59,7 @@ MooseX::Storage::Engine::Trait::WithRoles - An engine trait to include roles in
 
 =head1 VERSION
 
-version 0.1.0
+version 0.2.0
 
 =head1 AUTHOR
 
